local sprite = app.activeSprite
-- los colores en hex
local tabla_colores_hex = {}
local tabla_colores_rgb = {}

-- Check constraints
if sprite == nil then
  app.alert( "No Sprite..." )
  return
end

if sprite.colorMode ~= ColorMode.INDEXED then
  app.alert( "Sprite needs to be indexed" )
  return
end

--
--	FUNCIONES
--

local function inicializaTablas()
	-- numero de color que amstrad asigna a cada color
	-- https://www.cpcwiki.eu/index.php/CPC_Palette
	
	tabla_colores_rgb[ "color0" ] = Color{ r = 0, g = 0, b = 0 }
	tabla_colores_rgb[ "color1" ] = Color{ r = 0, g = 0, b = 128 }
	tabla_colores_rgb[ "color2" ] = Color{ r = 0, g = 0, b = 255 }
	
	tabla_colores_rgb[ "color3" ] = Color{ r = 128, g = 0, b = 0 }
	tabla_colores_rgb[ "color4" ] = Color{ r = 128, g = 0, b = 128 }
	tabla_colores_rgb[ "color5" ] = Color{ r = 128, g = 0, b = 255 }
	
	tabla_colores_rgb[ "color6" ] = Color{ r = 255, g = 0, b = 0 }
	tabla_colores_rgb[ "color7" ] = Color{ r = 255, g = 0, b = 128 }
	tabla_colores_rgb[ "color8" ] = Color{ r = 255, g = 0, b = 255 }
	
	tabla_colores_rgb[ "color9" ] = Color{ r = 0, g = 128, b = 0 }
	tabla_colores_rgb[ "color10" ] = Color{ r = 0, g = 128, b = 128 }
	tabla_colores_rgb[ "color11" ] = Color{ r = 0, g = 128, b = 255 }
	
	tabla_colores_rgb[ "color12" ] = Color{ r = 128, g = 128, b = 0 }
	tabla_colores_rgb[ "color13" ] = Color{ r = 128, g = 128, b = 128 }
	tabla_colores_rgb[ "color14" ] = Color{ r = 128, g = 128, b = 255 }
	
	tabla_colores_rgb[ "color15" ] = Color{ r = 255, g = 128, b = 0 }
	tabla_colores_rgb[ "color16" ] = Color{ r = 255, g = 128, b = 128 }
	tabla_colores_rgb[ "color17" ] = Color{ r = 255, g = 128, b = 255 }
	
	tabla_colores_rgb[ "color18" ] = Color{ r = 0, g = 255, b = 0 }
	tabla_colores_rgb[ "color19" ] = Color{ r = 0, g = 255, b = 128 }
	tabla_colores_rgb[ "color20" ] = Color{ r = 0, g = 255, b = 255 }
	
	tabla_colores_rgb[ "color21" ] = Color{ r = 128, g = 255, b = 0 }
	tabla_colores_rgb[ "color22" ] = Color{ r = 128, g = 255, b = 128 }
	tabla_colores_rgb[ "color23" ] = Color{ r = 128, g = 255, b = 255 }
	
	tabla_colores_rgb[ "color24" ] = Color{ r = 255, g = 255, b = 0 }
	tabla_colores_rgb[ "color25" ] = Color{ r = 255, g = 255, b = 128 }
	tabla_colores_rgb[ "color26" ] = Color{ r = 255, g = 255, b = 255 }
	
	-- en modo 0 puedes tener una paleta de 16 colores.
	-- el indice se refiere al indice de la paleta, cada color tiene un numero de ndice y segn ese nmero, le pertenece un valor u otro, no es segn el color, si no, segn su nmero de ndice. Un rojo en el ndice 1 ser valor 64, y si se cambia por un verde seguir teniendo un valor de 64 ya que es el ndice lo que impone el valor.
	
	tabla_colores_hex[ "indice0" ] = 0 		--00 valor en hex
	tabla_colores_hex[ "indice1" ] = 64 	--40
	tabla_colores_hex[ "indice2" ] = 4 		--04
	
	tabla_colores_hex[ "indice3" ] = 68 	--44
	tabla_colores_hex[ "indice4" ] = 16 	--10
	tabla_colores_hex[ "indice5" ] = 80 	--50
	
	tabla_colores_hex[ "indice6" ] = 20 	--14
	tabla_colores_hex[ "indice7" ] = 84 	--54
	tabla_colores_hex[ "indice8" ] = 1 		--01
	
	tabla_colores_hex[ "indice9" ] = 65 	--41
	tabla_colores_hex[ "indice10" ] = 5 	--05
	tabla_colores_hex[ "indice11" ] = 69 	--45
	
	tabla_colores_hex[ "indice12" ] = 17 	--11
	tabla_colores_hex[ "indice13" ] = 81 	--51
	tabla_colores_hex[ "indice14" ] = 21 	--15
	
	tabla_colores_hex[ "indice15" ] = 85 	--55
	--[[tabla_colores_hex[ "color16" ] = 3 	--03
	tabla_colores_hex[ "color17" ] = 4 	--04
	
	tabla_colores_hex[ "color18" ] = 5 	--05
	tabla_colores_hex[ "color19" ] = 7 	--07
	tabla_colores_hex[ "color20" ] = 11 --0B
	
	tabla_colores_hex[ "color21" ] = 15 --0F
	tabla_colores_hex[ "color22" ] = 17 --11
	tabla_colores_hex[ "color23" ] = 19 --13
	
	tabla_colores_hex[ "color24" ] = 21 --15
	tabla_colores_hex[ "color25" ] = 23 --17
	tabla_colores_hex[ "color26" ] = 25 --19--]]
end


-- escribe los datos de la paleta
local function writePaletteData( palette )
	local ncolors = #palette
	local texto = "Paleta:\n"

	-- recorre los primeros 15 colores de la paleta que tengamos definida en aseprite
	for i = 0, 15 do
		local color = palette: getColor( i )
		
		if color.alpha == 255 then
			
			-- recorre la tabla tabla_colores_rgb para encontrar qu valor de color se debe setear para el color actual i
			for b = 0, 26 do
				nombreColor = "color" .. tostring( b )
				
				-- si se encuentra la coincidencia se establece el valor de color
				if color.red == tabla_colores_rgb[ nombreColor ].red and 
				color.green == tabla_colores_rgb[ nombreColor ].green and 
				color.blue == tabla_colores_rgb[ nombreColor ].blue then
					local tinta = tostring( i )
					local numColor = tostring( b )
					texto = texto .. "ink " .. tinta .. "," .. numColor ..":"
				end
				
			end
			
		end
				
	end
	io.write( texto )
end


-- devuelve el valor de byte que corresponde a los dos pixeles recibidos en parametros
local function to_amstrad_palette( pixelIzq, pixelDer )
	local indicePixelIzq = "indice" .. tostring( pixelIzq )
	local indicePixelDer = "indice" .. tostring( pixelDer )

	-- muestra un error si el sprite tiene algn color que no se encuentra en la paleta
	if tabla_colores_hex[ indicePixelIzq ] == nil or tabla_colores_hex[ indicePixelDer ] == nil then
		local mensajeError = Dialog()
		
		mensajeError: label{
			id = "mensaje",
			label = "Error:",
			text = "Alguno de los colores usados en el sprite no coincide con los definidos en la paleta." }
		mensajeError:show()
	end
	
	local valorPixelIzq = tabla_colores_hex[ indicePixelIzq ] * 2
	local valorPixelDer = tabla_colores_hex[ indicePixelDer ]
	
	return( valorPixelIzq + valorPixelDer )
end


local function writePixelValues( img, x, y, w, h )
	-- escribe ancho y alto, los dos puntos .. sirven para concatenar strings. En esta funcin se concatena todo, en la variable "res", el resultado mostrado en el archivo de salida.
	local res = "db " .. tostring( math.floor( sprite.width / 2 ) ) .. "; ancho\n"
	res = res .. "db " .. tostring( math.floor( sprite.height ) ) .. "; alto\n"
	
	-- se recorre cada frame
	for y = 0, h - 1 do
		res = res .. "db "
		
		for x = 0, w - 1, 2 do
			-- se obtiene el valor del byte que contiene al pixel (ladrillo) actual y al siguiente
			valorPixel = to_amstrad_palette( img: getPixel( x, y ), img: getPixel( x + 1, y ) )
			-- ese valor se pasa a string para concatenarlo a px
			px = tostring( valorPixel ) --string.format( "%x", valorPixel )
			
			-- si no se ha llegado al borde de una fila, se pone una coma entre cada valor
			if x < w - 2 then
				px = px .. " , "
			end
			
			-- se concatena
			res = res .. px
		end
		
		-- se pasa a nueva lnea cuando se completa una fila
		res = res .. "\n"
	end

	return res
end


local function exportFrame( frm )
	if frm == nil then
		frm = 1
	end

	local img = Image( sprite.spec )
	img:drawSprite( sprite, frm )
	
	inicializaTablas()
	
	--io.write( "begin indices\n" )
	io.write( writePixelValues( img, x, y, sprite.width, sprite.height ) )
	--io.write( "end indices\n" )
	-- separacion entre cada frame
	io.write( "\n" )
end

--
--
--

-- crea objeto tipo cuadro de dialogo, ser el que muestra las opciones de exportacin
local dlg = Dialog()

-- se configuran las opciones que incorporar el cuadro de dialogo dlg
dlg:file{ 	id = "exportFile",
			label = "File",
			title = "8BP Exporter",
			open = false,
			save = true,
			-- comentar para que el usuario deba elegir archivo obligatoriamente
			filename= "8BPExport.txt",
			filetypes = { "txt" } }
			
dlg:check{ 	id = "onlyCurrentFrame",
			text = "Export only current frame",
			selected = false }
		   
dlg:check{ 	id = "exportPalette",
			text = "Export palette values",
			selected = true }
			
dlg:button{ id = "ok", text = "OK" }
dlg:button{ id = "cancel", text = "Cancel" }
dlg:show()

-- obtiene las opciones elegidas en el cuadro de dialogo dlg
local data = dlg.data

-- si el boton con id "ok" es presionado, se procede a exportar
if data.ok then
	local f = io.open( data.exportFile, "w" )
	io.output( f )
	
	-- si se selecciono no exportar todos los frames, slo se exporta el actual seleccionado
	if data.onlyCurrentFrame then
		exportFrame( app.activeFrame )
	
	-- se exportan todos los frames de la animacin
	else
	
	 	for i = 1, #sprite.frames do
	 		io.write( string.format( ";Frame %d\n", i ) )
	 		exportFrame( i )
		end
		
	end
	
	-- escribe datos de paleta
	if data.exportPalette then
		writePaletteData( sprite.palettes[ 1 ] )
		
	end

	io.close( f )
end

